import pygame

from gui.text import TextStyle
from . import CharacterRenderer


class BasicCharacterRenderer(CharacterRenderer):

    def __init__(self, style: TextStyle):
        fontSize = int(style.tileHeight * 1.3)
        for iterations in range(100):
            font = pygame.font.Font(style.fontName, fontSize)
            if style.flags & TextStyle.ITALIC:
                font.set_italic(True)
            if style.flags & TextStyle.BOLD:
                font.set_bold(True)
            surface = font.render("M", False, style.color)
            if surface.get_height() == style.tileHeight:
                break
            elif surface.get_height() > style.tileHeight:
                fontSize -= 1
            else:
                fontSize += 1
        self.__font = font
        self.__color = style.color
        self.__tileDefaultWidth = surface.get_width()  # type: int
        self.__tileDefaultHeight = surface.get_height()  # type: int

    @property
    def tileDefaultWidth(self) -> int:
        return self.__tileDefaultWidth

    @property
    def tileDefaultHeight(self) -> int:
        return self.__tileDefaultHeight

    def getCharacterAdvance(self, character: str) -> int:
        metrics = self.__font.metrics(character)
        return metrics[0][4]

    def render(self, character: str) -> pygame.Surface:
        return self.__font.render(character, False, self.__color)
