import logging

from gui import GUIFacadeFactory
from gui.input import KeyEvent, Keyboard
from gui.text import TextStyle, TextTokenizer
from misc import LevelLoader

logging.basicConfig(level=logging.INFO, format='\r%(filename)s:%(lineno)d: %(message)s')

# Create window
guiFacade = GUIFacadeFactory().createInstance("OpenGL")
guiFacade.createWindow(
    "OpenGL 2D Facade - https://www.patternsgameprog.com/",
    32*24, 32*18
)

# Characters layer
charsLayer = guiFacade.createCharactersLayer()
charsLayer.setTileset("assets/level/characters.png")
charsLayer.setTileSize(32, 32)
charsLayer.setCharacterCount(2)
charsLayer.setCharacterTile(0, 7, 0)
charsLayer.setCharacterLocation(0, 10.5, 6.0)
charsLayer.setCharacterTile(1, 4, 1)
charsLayer.setCharacterLocation(1, 12.5, 6.0)

# UI Layer
uiLayer = guiFacade.createUILayer()
uiLayer.setTileset("assets/ui/ui.png")
uiLayer.setTileSize(32, 32)
frameBox1 = uiLayer.createFrameBox(0, 100, 100, 200)  # For testing
frameBox2 = uiLayer.createFrameBox(32*1, 32*10, 32*23, 32*17)
frameBox3 = uiLayer.createFrameBox(0, 100, 100, 200)  # For testing
uiLayer.deleteFrameBox(frameBox1)  # For testing
uiLayer.deleteFrameBox(frameBox3)  # For testing

# Text Layer
textLayer = guiFacade.createTextLayer()
style = TextStyle("assets/font/terminal-grotesque.ttf", 32, (255, 255, 255))
textLayer.setStyle(style)
textLayer.setMaxCharacterCount(300)
textLayer.setText(
    32*1 + 16, 32*10 + 16,
    "<b>Hero:</b> Hello soldier! Can you tell me where I am ?\n"
    "<b>Soldier:</b> This example illustrates frame boxes in\n"
    "the OpenGL 2D Facade ! This facade allows to\n"
    "create a 2D game with Python, Pygame and OpenGL.\n"
    "Visit <u>https://www.patternsgameprog.com</u> for\n"
    "description, code and assets!"
)
messageCurrentIndex = 0
textLayer.setDisplayedCharacterCount(messageCurrentIndex)

# Main loop
guiFacade.init()
frameCount = 0
while not guiFacade.closingRequested:
    # Update input state
    guiFacade.updateInputs()

    # Keyboard key single press
    keyboard = guiFacade.keyboard
    for keyEvent in keyboard.keyEvents:
        if keyEvent.type == KeyEvent.KEYDOWN:
            if keyEvent.key == Keyboard.K_ESCAPE:
                guiFacade.closingRequested = True
                break
    keyboard.clearKeyEvents()

    # Update text every 3 frames
    if (frameCount % 3) == 0:
        textLayer.setDisplayedCharacterCount(messageCurrentIndex)
        messageCurrentIndex += 1
        if messageCurrentIndex >= 500:
            messageCurrentIndex = 0

    # Render scene
    guiFacade.render()
    frameCount += 1

if False:  # Screen capture (to check that the rendering is ok)
    capture = guiFacade.captureScreenContent(0, 0, guiFacade.screenWidth, guiFacade.screenHeight)
    Image.fromarray(capture).save("capture.png")

# Delete all objects and quit
guiFacade.quit()
